<?php
/**
* @package SKYPANTHER
* @version $Id: FileWriter.php,v 1.0 2006/01/13 12:30:00 tim Exp $
* @author Tim Poulsen, Skypanther Studios
*/
/**
* File Writer Class
* @access public
*/
class FileWriter {
    /**
    * Name of the file to be written
    * @access private
    * @var string
    */
	var $filename;

    /**
    * Full path to the location where the file will be written
    * @access private
    * @var string
    */
    var $fullpath;

    /**
    * File access mode (read, write, append)
    * @access private
    * @var string
    */
    var $mode;

    /**
    * File contents
    * @access private
    * @var string
    */
    var $filecontents;

    /**
    * Stores error messages
    * @access private
    * @var string
    */
    var $error;

    /**
    * FileWriter constructor
    * @param string filename (name of file to be written)
    * @param string filecontents (contents to be written)
    * @param string mode (mode: write or append)
    * @param string fullpath (path or auto, which forces this class to figure it out)
    * @access public
    */
    function FileWriter ($filename,$filecontents,$fullpath='auto',$mode='write') {
		$this->filename=$filename;
        $this->filecontents=$filecontents;
        $this->setFullPath($fullpath);
        $this->setMode($mode);
		$this->writefile();
	} // end FileWriter() constructor
	
    /**
    * setMode - sets the mode using the proper fopen syntax
    * @access private
    */
	function setMode ($mode) {
		switch($mode) {
		case 'write':
		case 'w':
		case 'w+':
			$this->mode = 'w+';
			break;
		case 'append':
		case 'a':
		case 'a+':
			$this->mode = 'a+';
			break;
		default:
			$this->error = 'Invalid file mode supplied. Valid values are: write, append, w, w+, a, and a+';
		}
	} // end setMode()
	
    /**
    * setFullPath - sets the full path to the file to be written
    * @access private
    */
	function setFullPath ($fullpath) {
		if($fullpath == 'auto') {
			// determine full path
			$this->fullpath = realpath('.');
		} else {
			$this->fullpath = $fullpath;
		}
		$this->fullpath = str_replace('\\\\', '/', $this->fullpath);
		$this->fullpath = rtrim($this->fullpath, '/');
	} // end setFullPath()

    /**
    * writefile - writes the file
    * @access private
    */
	function writefile () {
		$fp = fopen($this->fullpath.'/'.$this->filename,$this->mode);
		if (fwrite($fp, $this->filecontents) === FALSE) {
			$this->error = "Cannot write to file ({$this->fullpath}/{$this->filename})";
		}
	} // end setFullPath()


    /**
    * Returns filename, minus path
    * @return string
    * @access public
    */
    function getFilename () {
        return $this->filename;
    }

    /**
    * Returns path
    * @return string
    * @access public
    */
    function getPath () {
        return $this->fullpath;
    }

    /**
    * Returns filename including path
    * @return string
    * @access public
    */
    function getFilespec () {
        return $this->fullpath.'/'.$this->filename;
    }

    /**
    * Returns error messages
    * @return string
    * @access public
    */
    function getError () {
        return $this->error;
    }


} // end FileWriter class

?>