<?php
/* **************************************************
Copyright (c) 2008, Skypanther(r) Studios, Inc.
Skypanther(r) is a registered trademark of Skypanther Studios, Inc. 
************************************************** */

class ImageUpload {
	var $save_dir;						//where file will be saved
	var $filename="noimage.gif";        //default file name initially
	var $thumbnail = 'noimage.gif';		// default thumbnail image name
	var $error_message="";				//string to be output if neccesary
	var $width;							//height of final image
	var $height;						//width of final image
	var $thumb_width;					// width of thumbnail version
	var $thumb_height;					// height of thumbnail version
	
	function ImageUpload($save_directory, $file_array, $max_width='1024', $max_height='768', $thumb_width='125') {
	   $this->save_dir = $save_directory;               
	   $this->width =    $max_width;
	   $this->height =  $max_height;
	   $this->thumb_width = $thumb_width;
	   $this->error_message = '';
	
	   //--change filename to time - make it unique
	   $temp_filename = $file_array['name'];
	   $ext = explode('.',$temp_filename);
	   $ext = $ext[count($ext)-1];
	   $temp_filename = time();
	   $temp_thumbnail = $temp_filename."_th.".$ext;
	   $temp_filename = $temp_filename.".".$ext;


	   //--check that it's a jpeg or gif
	   if (preg_match('/^(gif|jpg|jpe|png?g)$/',strtolower($ext))) {
		   // resize in proportion
		   list($width_orig, $height_orig) = getimagesize($file_array['tmp_name']);
		   if(is_nan($width_orig) || $width_orig == 0) {
		   		$width_orig = 1;
				$this->error_message .= 'Invalid image width, bad image?<br />';
		   }
		   if(is_nan($height_orig) || $height_orig == 0) {
		   		$height_orig = 1;
				$this->error_message .= 'Invalid image height, bad image?<br />';
		   }
		   if ($this->width && ($width_orig < $height_orig)) {
				$this->width = ($this->height / $height_orig) * $width_orig;
		   } else {
			   $this->height = ($this->width / $width_orig) * $height_orig;
		   }

			// calculate thumbnail size in proportion
			$this->thumb_height = round(($this->thumb_width / $width_orig) * $height_orig);

			//handle gifs and jpegs separately
			if($ext=='gif'){
		   		// CREATING A GIF IMAGE AND ITS THUMBNAIL
				$image = imagecreatefromgif($file_array['tmp_name']);
				$image_p = imagecreatetruecolor($this->width, $this->height); 
				$thumb_p = imagecreatetruecolor($this->thumb_width, $this->thumb_height);

				// set transparency for the large image
				imagealphablending($image_p, false);
				imagesavealpha($image_p,true);
				$transparent = imagecolorallocatealpha($image_p, 255, 255, 255, 127);
				imagefilledrectangle($image_p, 0, 0, $this->width, $this->height, $transparent);
				// set transparency for the thumbnail
				imagealphablending($thumb_p, false);
				imagesavealpha($thumb_p,true);
				$transparent = imagecolorallocatealpha($thumb_p, 255, 255, 255, 127);
				imagefilledrectangle($thumb_p, 0, 0, $this->thumb_width, $this->thumb_height, $transparent);

				imagecopyresampled($image_p, $image, 0, 0, 0, 0, $this->width, $this->height, $width_orig, $height_orig);
				imagecopyresampled($thumb_p, $image, 0, 0, 0, 0, $this->thumb_width, $this->thumb_height, $width_orig, $height_orig);
				imagegif($image_p, $this->save_dir.$temp_filename);
				imagegif($thumb_p, $this->save_dir.$temp_thumbnail);
			} elseif($ext=='png') {
		   		// CREATING A PNG IMAGE AND ITS THUMBNAIL
				$image = @imagecreatefrompng($file_array['tmp_name']);
				$image_p = imagecreatetruecolor($this->width, $this->height); 
				$thumb_p = imagecreatetruecolor($this->thumb_width, $this->thumb_height);

				// set transparency for the large image
				imagealphablending($image_p, false);
				imagesavealpha($image_p,true);
				$transparent = imagecolorallocatealpha($image_p, 255, 255, 255, 127);
				imagefilledrectangle($image_p, 0, 0, $this->width, $this->height, $transparent);
				// set transparency for the thumbnail
				imagealphablending($thumb_p, false);
				imagesavealpha($thumb_p,true);
				$transparent = imagecolorallocatealpha($thumb_p, 255, 255, 255, 127);
				imagefilledrectangle($thumb_p, 0, 0, $this->thumb_width, $this->thumb_height, $transparent);

				if($image != '') {                           
					imagecopyresampled($image_p, $image, 0, 0, 0, 0, $this->width, $this->height, $width_orig, $height_orig);
					imagepng($image_p, $this->save_dir.$temp_filename);
					//$thumb = imagecreatefromjpeg($file_array['tmp_name']);                           
					imagecopyresampled($thumb_p, $image, 0, 0, 0, 0, $this->thumb_width, $this->thumb_height, $width_orig, $height_orig);
					imagepng($thumb_p, $this->save_dir.$temp_thumbnail);
				} else {
					$this->error_message .= 'Invalid or bad PNG image. Please try uploading a different file.<br />';
				}
			} else {
			// CREATING A JPG IMAGE AND ITS THUMBNAIL
				$image = imagecreatefromjpeg($file_array['tmp_name']);
				$image_p = imagecreatetruecolor($this->width, $this->height); 
				$thumb_p = imagecreatetruecolor($this->thumb_width, $this->thumb_height);
				if($image != '') {                           
					imagecopyresampled($image_p, $image, 0, 0, 0, 0, $this->width, $this->height, $width_orig, $height_orig);
					imagejpeg($image_p, $this->save_dir.$temp_filename, 80);
					//$thumb = imagecreatefromjpeg($file_array['tmp_name']);                           
					imagecopyresampled($thumb_p, $image, 0, 0, 0, 0, $this->thumb_width, $this->thumb_height, $width_orig, $height_orig);
					imagejpeg($thumb_p, $this->save_dir.$temp_thumbnail, 80);
				} else {
					$this->error_message .= 'Invalid or bad JPG image. Please try uploading a different file.<br />';
				}
			} // end gif/jpg block
	
		   @imagedestroy($image_p);
		   @imagedestroy($thumb_p);
		   @imagedestroy($image);
	
		   $this->filename=$temp_filename;
		   $this->thumbnail = $temp_thumbnail;

		} else {
			$this->error_message.="The file is not a JPG/JPEG, GIF, or PNG picture. Please upload a file of the appropriate type.";
		}
	} // end constructor function
} // end ImageUpload()
?>