<?php

	/*

		Credit CArd Validator
		Date - Jan 14, 2005
		Author - Harish Chauhan

		From PHPClasses, modified by Tim with code from a Zend class that didn't work and my own code
		
		Added: clean($ccnum) : strips non-numeric characters
		Added: encrypt($ccnum) : encrypts for secure transfer to process.php page
		Added: decrypt($ccnum) : decrypts encrypted cc numbers
		Added: hide($ccnum) : creates obfuscated number for display on confirmation page
		Added: identify($ccnum) : given a ccnum, returns the cc type (string)
		Added: _hex2bin($ccnum) : private, helper method used by decrypt()


	*/

	class CCValidate
	{
	
		function clean ($ccnum) {
			// Remove non-numeric characters from $ccnum 
			return ereg_replace ('[^0-9]+', '', $ccnum);
		} // end clean()
	

		function encrypt ($ccnum) {
			// Reversibly encrypts number 
			return bin2hex($ccnum);
		} // end encrypt()

		function decrypt ($ccnum) {
			// Decrypts number 
			return $this->_hex2bin($ccnum);
		} // end decrypt()

		function hide ($ccnum) {
			// Creates obfuscated number for output on page
			$num = '';
			for($i=0;$i<strlen($ccnum)-4;$i++) {
				$num .= '*';
			}
			return $num.substr($ccnum, -4);
		} // end hide()

		function _hex2bin($hexdata) {
			$bindata="";
			$cnt = strlen($hexdata);	
			for ($i=0;$i<$cnt;$i+=2) {
				$bindata.=chr(hexdec(substr($hexdata,$i,2)));
			}
			return $bindata;
		} // end hex2bin

		function identify ($ccnum)
		{
			 $ccnum = $this->clean ($ccnum);
	
			// Get card type based on prefix and length of card number 
			if (ereg ('^4(.{12}|.{15})$', $ccnum))
				return 'Visa'; 
			if (ereg ('^5[1-5].{14}$', $ccnum))
				return 'Mastercard';
			if (ereg ('^3[47].{13}$', $ccnum))
				return 'American Express';
			if (ereg ('^3(0[0-5].{11}|[68].{12})$', $ccnum))
				return 'Diners Club/Carte Blanche';
			if (ereg ('^6011.{12}$', $ccnum))
				return 'Discover Card';
			if (ereg ('^(3.{15}|(2131|1800).{11})$', $ccnum))
				return 'JCB';
			if (ereg ('^2(014|149).{11})$', $ccnum))
				return 'enRoute';
	
			return 'unknown';
		} // end identify


		function _checkSum($ccnum)
		{
			$checksum = 0;
			for ($i=(2-(strlen($ccnum) % 2)); $i<=strlen($ccnum); $i+=2)
			{
				$checksum += (int)($ccnum{$i-1});
			}
			// Analyze odd digits in even length strings or even digits in odd length strings.
		    for ($i=(strlen($ccnum)% 2) + 1; $i<strlen($ccnum); $i+=2) 
			{
			  $digit = (int)($ccnum{$i-1}) * 2;
			  if ($digit < 10) 
				  { $checksum += $digit; } 
			  else 
				  { $checksum += ($digit-9); }
		    }
			if (($checksum % 10) == 0) 
				return true; 
			else 
				return false;

		}

		function isValid($ccnum,$type="",$returnobj=false)
		{
			$ccnum = $this->clean($ccnum);
			$creditcard=array(  "visa"=>"/^4\d{3}-?\d{4}-?\d{4}-?\d{4}$/",
								"mastercard"=>"/^5[1-5]\d{2}-?\d{4}-?\d{4}-?\d{4}$/",
								"discover"=>"/^6011-?\d{4}-?\d{4}-?\d{4}$/",
								"amex"=>"/^3[4,7]\d{13}$/",
								"diners"=>"/^3[0,6,8]\d{12}$/",
								"bankcard"=>"/^5610-?\d{4}-?\d{4}-?\d{4}$/",
								"jcb"=>"/^[3088|3096|3112|3158|3337|3528]\d{12}$/",
								"enroute"=>"/^[2014|2149]\d{11}$/",
								"switch"=>"/^[4903|4911|4936|5641|6333|6759|6334|6767]\d{12}$/");
			if(empty($type))
			{
				$match=false;
				foreach($creditcard as $type=>$pattern)
					if(preg_match($pattern,$ccnum)==1)
					{
						$match=true;
						break;
					}

				if(!$match)
					return false;
				else
				{
					if($returnobj)
					{
						$return=new stdclass;
						$return->valid=$this->_checkSum($ccnum);
						$return->ccnum=$ccnum;
						$return->type=$type;
						return $return;
					}
					else
						return $this->_checkSum($ccnum);
				}
			
			}
			else
			{
				if(@preg_match($creditcard[strtolower(trim($type))],$ccnum)==0)
					return false;
				else
				{
					if($returnobj)
					{
						$return=new stdclass;
						$return->valid=$this->_checkSum($ccnum);
						$return->ccnum=$ccnum;
						$return->type=$type;
						return $return;
					}
					else
						return $this->_checkSum($ccnum);
				}
			}
		}
}
?>