/* *************************************************************
* General purpose form validator, by Tim Poulsen               *
*                                                              *
* Will check for entry in required fields, check for numeric   *
* data, minimum and maximum values, phone number and zip       *
* codes, and two-letter state abbreviations                    *
*                                                              *
* Include this function with the following statement           *
* <SCRIPT LANGUAGE="JavaScript" SRC="formval.js"></SCRIPT>     *
*                                                              *
* Function expects form elements to have properties            *
* added in the HTML code, like this:                           *
*                                                              *
* <FORM onSubmit = "                                           *
*   this.lastname.required=true;                               *
*   this.age.isNumeric=true;                                   *
*   this.age.minVal=18;                                        *
*   this.age.maxVal=100;                                       *
*   this.phonenumber.phone = true;                             *
*   this.zipcode.zip = true;                                   *
*   this.state.state = true;                                   *
*   return validate(this);                                     *
* ">                                                           *
*                                                              *
***************************************************************/

/* *************************************************************
* set some global variables that will be checked later         *
***************************************************************/
var msg = "";                 // an output message
var missing = "";             // for missing required fields
var invNum = "";              // for invalid numeric fields
var outOfRange = "";          // less than min or more than max
var invZIP = "";              // for invalid zip codes
var invPhone = "";            // for invalid phone numbers
var invState = "";            // for invalid state fields
  
/* *************************************************************
* The main validation function, calls other sub-functions      *
***************************************************************/
function validate(frm)
  {
  for(i=0; i<frm.elements.length; i++)            // loop through form elements
    {
    var el = frm.elements[i];
    if(el.required)                               // if element has required property
      {                                           // test to see if field is empty
      if(isEmpty(el))
        {
        missing += "\n   - " + el.name + " is a required field";
        }
      }
    if(el.isNumeric)
      {
      if(!isNumeric(el))
        {
        invNum += "\n   - In " + el.name + ", " + el.value + " is not a number";
        }
      }
    if(el.minVal)
      {
      if(parseFloat(el.value) <= el.minVal)
        {
        outOfRange +=  "\n   - " + el.name + " must be larger than " + el.minVal + ", you entered " + el.value;
        }
      }
    if(el.maxVal)
      {
      if(parseFloat(el.value) >= el.maxVal)
        {
        outOfRange +=  "\n   - " + el.name + " must be smaller than " + el.maxVal + ", you entered " + el.value
        }
      }
    if(el.zip && el.value.length !=0)
      {
      if(invalidZIP(el))
        {
        invZIP += "\n  - " + el.value + " is an invalid ZIP code";
        }
      }
    if(el.phone && el.value.length !=0)
      {
      if(invalidPhone(el))
        {
        invPhone += "\n  - " + el.value + " is not a valid phone number";
        }
      }
    if(el.state && el.value.length != 0)
      {
      if(invalidState(el))
        {
        invState += "\n  - " + el.value + " is not a valid two-letter state abbreviation";
        }
      }
    }
  
  // build output message
  if(missing.length !=0 || invNum.length != 0 || outOfRange.length != 0 || invZIP.length != 0 || invPhone.length != 0 || invState.length != 0)
    {
    if(missing.length !=0)
      {
      msg += "\n\nThe following required fields are missing:";
      msg += missing;
      }
    if(invNum.length !=0)
      {
      msg += "\n\nYou entered incorrect numeric data in these fields:";
      msg += invNum;
      }
    if(outOfRange.length !=0)
      {
      msg += "\n\nYou entered out-of-range data in these fields:";
      msg += outOfRange;
      }
    if(invZIP.length !=0)
      {
      msg += "\n\nYou entered an incorrect ZIP code in these fields:";
      msg += invZIP;
      }
    if(invPhone.length !=0)
      {
      msg += "\n\nYou entered an incorrect phone number in these fields:";
      msg += invPhone;
      }
    if(invState.length !=0)
      {
      msg += "\n\nYou entered an incorrect state abbreviation";
      msg += invState;
      }
    errMsg(msg);           // call the output function
    msg = ""; missing = ""; invNum = ""; invZIP = ""; invPhone = ""; invState = "" ; outOfRange = "" // reset all our variables
    return false;
    }
  else
    {
    return true;
    }
  }

/* *************************************************************
* Sub-functions followFROM ".$config['dbprefix']."here to end of file                *
* All sub-functions return true if field is of invalid         *
* format and false if they are valid entries                   *
***************************************************************/
function isEmpty(field)
  {
  str = field.value;
  if(str == "") 
    {
    return true;
    }
  else
    {
    for(j=0; j<str.length; j++)
      {
      if(str.charAt(j) != " ") return false;
      }
    }
  return true;
  }

function isNumeric(field)
  {
  var errCount = 0;
  var numdecs = 0;              // number of decimal points
  var nummins = 0;              // number of minus signs
  for(j=0;j<field.value.length;j++)
    {
    c = field.value.charAt(j);        // short hand notation for character at position j
    if((c >= 0 && c <= 9) || (c=="." || c=="-"))
      {
      if(c==".") numdecs++;     // count the number of decimal points
      if(c=="-") nummins++;     // count the number of minus signs
      }
    else
      {
      errCount++;               // if it's none of those, increment error counter
      }
    }
  // error if count is non-zero or there are more than one decimal point or minus sign
  if(errCount > 0 || numdecs > 1 || nummins > 1)
    {
    return false;
    }
  return true;
  }

function stripNonDigits(str)
  {
  newStr = "";
  for(j=0; j<str.length; j++)
    {
    c = str.charAt(j);
    if(c >= "0" && c <= "9")
      {
      newStr += c;
      }
    }
  return newStr;
  }

function invalidZIP(field)
  {
  newStr = stripNonDigits(field.value);
  if(newStr.length == 5 || newStr.length== 9)
    {
    return false;
    }
  return true;
  }

function invalidPhone(field)
  {
  newStr = stripNonDigits(field.value);
  if(newStr.length == 10)
    {
    return false;
    }
  return true;
  }

function invalidState(field)
  {
  var STATES = "AL/AK/AZ/AR/CA/CO/CT/DE/DC/FL/GA/HI/ID/IL/IN/IA/KS/LA/ME/MD/MA/MI/MN/MS/MO/MT/NV/NH/NJ/NM/NY/NC/ND/OH/OK/OR/PA/PR/RI/SC/TN/TX/UT/VT/VA/WA/WV/WI/WY";
  var newStr = field.value.toUpperCase();
  if(STATES.indexOf(newStr) == -1 || newStr.indexOf("/") != -1 || newStr.length != 2)
    {
    return true;
    }
  return false;
  }

function errMsg(msg)
  {
  var theMsg = "You entered some incorrect values into the form. ";
  theMsg += "Please correct your entries then re-submit the form.\n";
  theMsg += "____________________________________________________________________";
  theMsg += msg;
  theMsg += "\n____________________________________________________________________\n";
  alert(theMsg);
  }
